/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.service;

import com.sun.istack.NotNull;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.extractor.model.Branch;
import ru.ulstu.extractor.model.Commit;
import ru.ulstu.extractor.model.Repository;
import ru.ulstu.extractor.repository.BranchRepository;
import ru.ulstu.extractor.repository.CommitRepository;
import ru.ulstu.extractor.repository.RepositoryRepository;

import java.io.IOException;
import java.util.List;

@Service
public class IndexService {
    private final GitRepositoryService gitRepositoryService;
    private final RepositoryRepository repositoryRepository;
    private final BranchRepository branchRepository;
    private final CommitRepository commitRepository;

    public IndexService(GitRepositoryService gitRepositoryService,
                        RepositoryRepository repositoryRepository,
                        BranchRepository branchRepository,
                        CommitRepository commitRepository) {
        this.gitRepositoryService = gitRepositoryService;
        this.repositoryRepository = repositoryRepository;
        this.branchRepository = branchRepository;
        this.commitRepository = commitRepository;
    }

    @Transactional
    public void index(@NotNull String repositoryUrl, @NotNull String branchName) throws GitAPIException, IOException {
        Repository repository = repositoryRepository.findByUrl(repositoryUrl);
        if (repository == null) {
            repository = repositoryRepository.save(new Repository(repositoryUrl));
        }
        Branch branch = branchRepository.findByRepositoryAndName(repository, branchName);
        if (branch == null) {
            branch = branchRepository.save(new Branch(repository, branchName));
        }
        List<Commit> commits = gitRepositoryService.getCommits(repositoryUrl, branchName);
        List<Commit> commitsToRemove = branch.getCommits();
        branch.getCommits().clear();
        commitRepository.deleteAll(commitsToRemove);
        branch.setCommits(commits);
        branchRepository.save(branch);
    }
}
